@@ -7,16 +7,20 @@ from threading import Event
 
 # pylint: disable=wildcard-import
 import threading
+
+import zeroconf
+
 from .config import *  # noqa
 from .error import *  # noqa
 from . import socket_client
-from .discovery import (
+from .discovery import (  # noqa
     DISCOVER_TIMEOUT,
+    CastListener,
     discover_chromecasts,
     start_discovery,
     stop_discovery,
 )
-from .dial import get_device_status, reboot, DeviceStatus
+from .dial import get_device_status, DeviceStatus
 from .const import CAST_MANUFACTURERS, CAST_TYPES, CAST_TYPE_CHROMECAST
 from .controllers.media import STREAM_TYPE_BUFFERED  # noqa
 
@@ -60,12 +64,14 @@ def get_chromecast_from_host(host, tries=None, retry_wait=None, timeout=None):
 _get_chromecast_from_host = get_chromecast_from_host  # pylint: disable=invalid-name
 
 
-def get_chromecast_from_service(services, tries=None, retry_wait=None, timeout=None):
+def get_chromecast_from_service(
+    services, zconf, tries=None, retry_wait=None, timeout=None
+):
     """Creates a Chromecast object from a zeroconf service."""
     # Build device status from the mDNS service name info, this
     # information is the primary source and the remaining will be
     # fetched later on.
-    services, zconf, uuid, model_name, friendly_name = services
+    services, uuid, model_name, friendly_name, _, _ = services
     _LOGGER.debug("_get_chromecast_from_service %s", services)
     cast_type = CAST_TYPES.get(model_name.lower(), CAST_TYPE_CHROMECAST)
     manufacturer = CAST_MANUFACTURERS.get(model_name.lower(), "Google Inc.")
@@ -100,13 +106,21 @@ def get_listed_chromecasts(
     retry_wait=None,
     timeout=None,
     discovery_timeout=DISCOVER_TIMEOUT,
+    zeroconf_instance=None,
 ):
     """
     Searches the network for chromecast devices matching a list of friendly
     names or a list of UUIDs.
 
-    Returns a list of discovered chromecast devices matching the criteria,
-    or an empty list if no matching chromecasts were found.
+    Returns a tuple of:
+      A list of Chromecast objects matching the criteria,
+      or an empty list if no matching chromecasts were found.
+      A service browser to keep the Chromecast mDNS data updated. When updates
+      are (no longer) needed, pass the browser object to
+      pychromecast.discovery.stop_discovery().
+
+    To only discover chromcast devices wihtout connecting to them, use
+    discover_listed_chromecasts instead.
 
     :param friendly_names: A list of wanted friendly names
     :param uuids: A list of wanted uuids
@@ -115,44 +129,72 @@ def get_listed_chromecasts(
     :param timeout: passed to get_chromecasts
     :param discovery_timeout: A floating point number specifying the time to wait
                                devices matching the criteria have been found.
+    :param zeroconf_instance: An existing zeroconf instance.
     """
 
-    cc_list = set()
+    cc_list = {}
 
-    def callback(chromecast):
-        _LOGGER.debug("Found chromecast %s", chromecast)
-        if uuids and chromecast.uuid in uuids:
-            cc_list.add(chromecast)
-            uuids.remove(chromecast.uuid)
-        elif friendly_names and chromecast.name in friendly_names:
-            cc_list.add(chromecast)
-            friendly_names.remove(chromecast.name)
-        if not friendly_names and not uuids:
-            discover_complete.set()
+    def callback(uuid, name):  # pylint: disable=unused-argument
+        _LOGGER.debug("Found chromecast %s", uuid)
+
+        def get_chromecast_from_uuid(uuid):
+            return get_chromecast_from_service(
+                listener.services[uuid],
+                zconf=zconf,
+                tries=tries,
+                retry_wait=retry_wait,
+                timeout=timeout,
+            )
+
+        service = listener.services[uuid]
+        friendly_name = service[3]
+        try:
+            if uuids and uuid in uuids:
+                if uuid not in cc_list:
+                    cc_list[uuid] = get_chromecast_from_uuid(uuid)
+                uuids.remove(uuid)
+            if friendly_names and friendly_name in friendly_names:
+                if uuid not in cc_list:
+                    cc_list[uuid] = get_chromecast_from_uuid(uuid)
+                friendly_names.remove(friendly_name)
+            if not friendly_names and not uuids:
+                discover_complete.set()
+        except ChromecastConnectionError:  # noqa
+            pass
 
     discover_complete = Event()
-    internal_stop = get_chromecasts(
-        tries=tries,
-        retry_wait=retry_wait,
-        timeout=timeout,
-        callback=callback,
-        blocking=False,
-    )
+
+    listener = CastListener(callback)
+    zconf = zeroconf_instance or zeroconf.Zeroconf()
+    browser = start_discovery(listener, zconf)
+
     # Wait for the timeout or found all wanted devices
     discover_complete.wait(discovery_timeout)
-    internal_stop()
-    return list(cc_list)
+    return (list(cc_list.values()), browser)
 
 
 # pylint: disable=too-many-locals
 def get_chromecasts(
-    tries=None, retry_wait=None, timeout=None, blocking=True, callback=None
+    tries=None,
+    retry_wait=None,
+    timeout=None,
+    blocking=True,
+    callback=None,
+    zeroconf_instance=None,
 ):
     """
-    Searches the network for chromecast devices and creates a Chromecast instance
+    Searches the network for chromecast devices and creates a Chromecast object
     for each discovered device.
 
-    May return an empty list if no chromecasts were found.
+    Returns a tuple of:
+      A list of Chromecast objects, or an empty list if no matching chromecasts were
+      found.
+      A service browser to keep the Chromecast mDNS data updated. When updates
+      are (no longer) needed, pass the browser object to
+      pychromecast.discovery.stop_discovery().
+
+    To only discover chromcast devices wihtout connecting to them, use
+    discover_chromecasts instead.
 
     Parameters tries, timeout, retry_wait and blocking_app_launch controls the
     behavior of the created Chromecast instances.
@@ -169,46 +211,55 @@ def get_chromecasts(
                      and returns a function which can be executed to stop discovery.
     :param callback: Callback which is triggerd for each discovered chromecast when
                      blocking = False.
+    :param zeroconf_instance: An existing zeroconf instance.
     """
     if blocking:
         # Thread blocking chromecast discovery
-        hosts = discover_chromecasts()
+        services, browser = discover_chromecasts()
         cc_list = []
-        for host in hosts:
+        for service in services:
             try:
                 cc_list.append(
-                    get_chromecast_from_host(
-                        host, tries=tries, retry_wait=retry_wait, timeout=timeout
+                    get_chromecast_from_service(
+                        service,
+                        browser.zc,
+                        tries=tries,
+                        retry_wait=retry_wait,
+                        timeout=timeout,
                     )
                 )
             except ChromecastConnectionError:  # noqa
                 pass
-        return cc_list
+        return (cc_list, browser)
 
     # Callback based chromecast discovery
     if not callable(callback):
         raise ValueError("Nonblocking discovery requires a callback function.")
 
-    def internal_callback(name):
+    known_uuids = set()
+
+    def internal_callback(uuid, name):  # pylint: disable=unused-argument
         """Called when zeroconf has discovered a new chromecast."""
+        if uuid in known_uuids:
+            return
         try:
             callback(
-                get_chromecast_from_host(
-                    listener.services[name],
+                get_chromecast_from_service(
+                    listener.services[uuid],
+                    zconf=zconf,
                     tries=tries,
                     retry_wait=retry_wait,
                     timeout=timeout,
                 )
             )
+            known_uuids.add(uuid)
         except ChromecastConnectionError:  # noqa
             pass
 
-    def internal_stop():
-        """Stops discovery of new chromecasts."""
-        stop_discovery(browser)
-
-    listener, browser = start_discovery(internal_callback)
-    return internal_stop
+    listener = CastListener(internal_callback)
+    zconf = zeroconf_instance or zeroconf.Zeroconf()
+    browser = start_discovery(listener, zconf)
+    return browser
 
 
 # pylint: disable=too-many-instance-attributes, too-many-public-methods
@@ -245,19 +296,17 @@ class Chromecast:
         timeout = kwargs.pop("timeout", None)
         retry_wait = kwargs.pop("retry_wait", None)
         services = kwargs.pop("services", None)
-        zconf = kwargs.pop("zconf", True)
+        zconf = kwargs.pop("zconf", None)
 
         self.logger = logging.getLogger(__name__)
 
         # Resolve host to IP address
         self._services = services
-        self.host = host
-        self.port = port or 8009
 
         self.logger.info("Querying device status")
         self.device = device
         if device:
-            dev_status = get_device_status(self.host, services, zconf)
+            dev_status = get_device_status(host, services, zconf)
             if dev_status:
                 # Values from `device` have priority over `dev_status`
                 # as they come from the dial information.
@@ -273,11 +322,11 @@ class Chromecast:
             else:
                 self.device = device
         else:
-            self.device = get_device_status(self.host, services, zconf)
+            self.device = get_device_status(host, services, zconf)
 
         if not self.device:
             raise ChromecastConnectionError(  # noqa
-                "Could not connect to {}:{}".format(self.host, self.port)
+                "Could not connect to {}:{}".format(host, port or 8009)
             )
 
         self.status = None
@@ -349,7 +398,7 @@ class Chromecast:
     @property
     def uri(self):
         """ Returns the device URI (ip:port) """
-        return "{}:{}".format(self.host, self.port)
+        return "{}:{}".format(self.socket_client.host, self.socket_client.port)
 
     @property
     def model_name(self):
@@ -402,12 +451,8 @@ class Chromecast:
 
         self.socket_client.receiver_controller.stop_app()
 
-    def reboot(self):
-        """ Reboots the Chromecast. """
-        reboot(self.host)
-
     def volume_up(self, delta=0.1):
-        """ Increment volume by 0.1 (or delta) unless it is already maxed.
+        """Increment volume by 0.1 (or delta) unless it is already maxed.
         Returns the new volume.
 
         """
@@ -418,7 +463,7 @@ class Chromecast:
         return self.set_volume(self.status.volume_level + delta)
 
     def volume_down(self, delta=0.1):
-        """ Decrement the volume by 0.1 (or delta) unless it is already 0.
+        """Decrement the volume by 0.1 (or delta) unless it is already 0.
         Returns the new volume.
         """
         if delta <= 0:
@@ -446,9 +491,9 @@ class Chromecast:
         self.status_event.wait(timeout=timeout)
 
     def connect(self):
-        """ Connect to the chromecast.
+        """Connect to the chromecast.
 
-            Must only be called if the worker thread will not be started.
+        Must only be called if the worker thread will not be started.
         """
         self.socket_client.connect()
 
@@ -491,14 +536,14 @@ class Chromecast:
 
     def __repr__(self):
         txt = "Chromecast({!r}, port={!r}, device={!r})".format(
-            self.host, self.port, self.device
+            self.socket_client.host, self.socket_client.port, self.device
         )
         return txt
 
     def __unicode__(self):
         return "Chromecast({}, {}, {}, {}, {})".format(
-            self.host,
-            self.port,
+            self.socket_client.host,
+            self.socket_client.port,
             self.device.friendly_name,
             self.device.model_name,
             self.device.manufacturer,
