@@ -12,6 +12,8 @@ APP_PLEX = "06ee44ee-e7e3-4249-83b6-f5d0b6f07f34_1"
 APP_DASHCAST = "84912283"
 APP_SPOTIFY = "CC32E753"
 APP_HOME_ASSISTANT = "B12CE3CA"
+APP_SUPLA = "A41B766D"
+APP_YLEAREENA = "A9BCCB7C"
 
 
 def get_possible_app_ids():
