@@ -53,6 +53,7 @@ def media_to_chromecast_command(
     autoplay=True,
     currentTime=0,
     playQueueID=None,
+    version="1.10.1.4602",
     **kwargs
 ):  # noqa: 501 pylint: disable=invalid-name, too-many-arguments, too-many-locals, protected-access, redefined-builtin
     """Create the message that chromecast requires. Use pass of plexapi media object or
@@ -80,6 +81,7 @@ def media_to_chromecast_command(
         username (None): user name of the person that start the playback.
         autoplay (bool): Auto play after the video is done.
         currentTime (int): Set playback from this time. default 0
+        version (str): pms version. Default 1.10.1.4602
         **kwargs: To allow overrides, this will be merged with the rest of the msg.
 
     Returns:
@@ -126,7 +128,7 @@ def media_to_chromecast_command(
                     "transcoderVideo": transcoderVideo,
                     "transcoderVideoRemuxOnly": transcoderVideoRemuxOnly,
                     "transcoderAudio": transcoderAudio,
-                    "version": "1.4.3.3433",
+                    "version": version,
                     "myPlexSubscription": myPlexSubscription,
                     "isVerifiedHostname": isVerifiedHostname,
                     "protocol": protocol,
@@ -223,8 +225,8 @@ class PlexController(BaseController):
         """Called when media channel is connected. Will update status."""
         self.update_status()
 
-    def receive_message(self, message, data):
-        """Called when a messag from plex to our controller is received.
+    def receive_message(self, message, data: dict):
+        """Called when a message from plex to our controller is received.
 
         I havnt seen any message for ut but lets keep for for now, the
         tests i have done is minimal.
@@ -232,7 +234,7 @@ class PlexController(BaseController):
 
         Args:
             message (dict): Description
-            data (dict): Description
+            data (dict): message.payload_utf8 interpreted as a JSON dict
 
         Returns:
             bool: True if the message is handled, False if not.
@@ -297,7 +299,7 @@ class PlexController(BaseController):
         )  # noqa: 501
 
     def volume_up(self, delta=0.1):
-        """ Increment volume by 0.1 (or delta) unless it is already maxed.
+        """Increment volume by 0.1 (or delta) unless it is already maxed.
         Returns the new volume.
         """
         if delta <= 0:
@@ -307,7 +309,7 @@ class PlexController(BaseController):
         return self.set_volume(self.status.volume_level + delta)
 
     def volume_down(self, delta=0.1):
-        """ Decrement the volume by 0.1 (or delta) unless it is already 0.
+        """Decrement the volume by 0.1 (or delta) unless it is already 0.
         Returns the new volume.
         """
         if delta <= 0:
@@ -408,14 +410,14 @@ class PlexController(BaseController):
     def block_until_playing(self, media=None, timeout=None, **kwargs):
         """Block until this playing, typically usefull in a script
 
-           another way to the the same is the check if the
-           controllers is_active or use self.status.player_state
+        another way to the the same is the check if the
+        controllers is_active or use self.status.player_state
 
-           Args:
-            media (None, optional): Can also be :class:`~plexapi.base.Playable
-                                   if its not, you need to fill out all the kwargs.
-            timeout (None, int): default None
-            **kwargs: See media_to_chromecast_command docs string.
+        Args:
+         media (None, optional): Can also be :class:`~plexapi.base.Playable
+                                if its not, you need to fill out all the kwargs.
+         timeout (None, int): default None
+         **kwargs: See media_to_chromecast_command docs string.
 
         """  # noqa
         # Incase media isnt playing
@@ -530,3 +532,16 @@ class PlexApiController(PlexController):
             subtitle (str): could be index, language or languageCode.
         """
         self._change_track(subtitle)
+
+    def play_media(self, media=None, **kwargs):
+        """Start playback on the chromecast
+
+        Args:
+            media (None, optional): Can also be :class:`~plexapi.base.Playable
+                                   if its not, you need to fill out all the kwargs.
+            **kwargs: See media_to_chromecast_command docs string. `version` is by default set to the
+                version of the PMS reported by the API.
+        """  # noqa
+        args = {"version": self.pms.version}
+        args.update(kwargs)
+        super().play_media(media, **args)
